



//  Structured data with shared behavior

struct Rect { width: f64, height: f64 } // structs store related data

impl Rect {
    fn new() -> Self {
        Rect { height: 1.0, width: 1.0 }
    }
}

trait Area { fn area(&self) -> f64; } // traits allow shared behavior

impl Area for Rect {
    fn area(&self) -> f64 { self.width * self.height }
}

#[derive(PartialEq, PartialOrd)]  // We derive traits from the std library for operator overloading
struct Circ(f64); // tuple struct

impl Area for Circ {
    fn area(&self) -> f64 { 2.0 * 3.14 * self.0 * self.0}
}

fn main() {
    let r = Rect::new(); // associated function as "constructor"
    let c1 = Circ(1.0);
    let c2 = Circ(42.0);
    if c2 > c1 {  // we can compare two circles because we derived the PartialOrd trait
        println!("{}, {}", r.area(), c2.area()); // method syntax equivalent to Area::area(&r)
    }
}
