// Generic Types

struct Rect<T> { width: T, height: T, }  // Generic types are annotated via <T>

impl<T> Rect<T> { // Generics are monomorphized during compile time
    fn new(a: T, b: T) -> Self { // nomomorphized functions are statically dispatched
        Rect { height: a, width: b }
    }
}

struct Circ<T>(T);

trait Area { fn area(&self) -> f64; }

impl<T> Area for Rect<T>
where T: Copy, f64: From<T>  // You can condition Generics with trait bounds
{                            // where Type1: Trait1 + Trait2, Type2: Trait3 + Trait4
    fn area(&self) -> f64 {
        f64::from(self.width) * f64::from(self.height)
    }
}

impl<T> Area for Circ<T>
where T: Copy, f64: From<T>
{
    fn area(&self) -> f64 {
        2.0 * 3.14 * f64::from(self.0) * f64::from(self.0)
    }
}

fn main() {
    let r = Rect::new(2, 3);  // Note that we do not have to specify the generic type explicitly!
    let c = Circ(42u8);
    println!("{}, {}", r.area(), c.area());
}
