



// Heap allocation


fn call_by_value(s: String) {
    println!("{}", s);
}

fn main() {

    let mut s = String::from("Hello,"); // Dynamically sized strings.
    s.push_str("world!");

    let mut v: Vec<i32> = vec![1, 2, 3, 4]; // Heap allocation for dynamically sized arrays.
    v.push(5);

    let one: Box<i32> = Box::new(1); // Heap allocation + smart pointer to that value.
    let two = 1 + *one; // You can dereference a Box as if it was a reference.

    let v2 = v; // Transfer of ownership invalidates previous variable.
    call_by_value(s); // Call-by-value will move ownership too.

    let v3 = v2.clone(); // Deep copy of heap data via clone method.

    // ⚡ Compile time error!
    // Trying to access invalidated variables
    println!("{}", v[0]);
    println!("{}", s);

    // All heap allocations are automatically cleaned up at the end of scope
    // No manual freeing of memory, no double free, no use after free
}
