



// Shared-State Concurrency


use std::thread;


// Instead we need mutually exclusive access (Mutex) and atomic reference counting (Arc)
fn main() {

    let mut state = Arc::new(Mutex::new(0u64));
    let mut handles = vec![];

    for _ in 0..2 {
		let state = Arc::clone(&state);
		let handle = thread::spawn(move || {
		    let mut n = state.lock().unwrap();  // via `n` we have exclusive access
		    *n += 1; // the type of `n` is a smart poniter to the inner value
		}); // When n goes out of scope, the lock is automatically lifted
		
		handles.push(handle);
	}

    for handle in handles {
        handle.join().unwrap();
    }
    println!("{}", *state.lock().unwrap());
}
